import {Component, Input, OnInit} from '@angular/core';
import {Item} from "../../../project/game-model/inventory/Item";
import {MatTableDataSource} from "@angular/material/table";
import {ItemProperty} from "../../../project/game-model/inventory/ItemProperty";
import {animate, state, style, transition, trigger} from "@angular/animations";
import {MatSnackBar} from "@angular/material/snack-bar";

@Component({
  selector: 'app-item-property-editor',
  templateUrl: './item-property-editor.component.html',
  styleUrl: './item-property-editor.component.scss',
  animations: [
    trigger('detailExpand', [
      state('collapsed,void', style({height: '0px', minHeight: '0'})),
      state('expanded', style({height: '*'})),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
    ]),
  ],
})
export class ItemPropertyEditorComponent implements OnInit{

  @Input() item: Item | undefined
  editedItemProperty: ItemProperty | undefined
  expandedElement: ItemProperty | undefined

  displayedColumns: string[] = ['name', 'property', 'edit', 'delete']
  columnsToDisplayWithExpand = [...this.displayedColumns, 'expand'];

  datasource: MatTableDataSource<ItemProperty> = new MatTableDataSource<ItemProperty>();

  constructor(private snackbar: MatSnackBar) {
  }

  ngOnInit() {
    this.datasource.data = this.item!.itemProperties
  }

  deleteItemProperty(itemProperty: ItemProperty) {
    this.item!.itemProperties = this.item!.itemProperties.filter(property => property.propertyName !== itemProperty.propertyName)
    this.datasource.data = this.item!.itemProperties
  }

  editItemProperty(itemProperty: ItemProperty) {
    this.editedItemProperty = itemProperty
  }

  finishEditing() {
    if(this.editedItemProperty!.propertyName.length < 1) {
      this.snackbar.open("PropertyName cannout be empty", "", {duration: 2000});
    } else {
      this.editedItemProperty = undefined
    }
  }

  addItemProperty() {
    const itemProperty = new ItemProperty("New Property", "", 0);
    this.item!.itemProperties.push(itemProperty);
    this.datasource.data = this.item!.itemProperties;
    this.editedItemProperty = itemProperty;
  }
}
