import {Transition} from "../transitions/Transition";
import {ScriptAccountCondition} from "../conditions/ScriptAccountCondition";
import {ScriptAccount} from "../../scriptAccounts/ScriptAccount";

export abstract class State<T extends Transition<any>> {

  incomingTransitions: T[] =[];
  outgoingTransitions: T[] =[];

  initial: boolean = false;

  conditions: ScriptAccountCondition[] = [];

  addIncomingTransition(transition: T) {
    this.incomingTransitions.push(transition);
  }

  addOutgoingTransition(transition: T) {
    this.outgoingTransitions.push(transition);
  }

  removeIncomingTransition(transition: T) {

  }

  removeOutgoingTransition(transition: T) {

  }

  addScriptAccountCondition(scriptAccountCondition: ScriptAccountCondition) {
    const existingScriptAccountCondition = this.findScriptAccountConditionByScriptAccount(scriptAccountCondition.scriptAccount);
    if(existingScriptAccountCondition != undefined) {
      if(!existingScriptAccountCondition.isContradicting(scriptAccountCondition)) {
        existingScriptAccountCondition.extendCondition(scriptAccountCondition)
      }
    } else {
      this.conditions.push(scriptAccountCondition);
    }
  }

  removeScriptAccountCondition(scriptAccount: ScriptAccount) {
    this.conditions = this.conditions.filter(condition => condition.scriptAccount !== scriptAccount);
  }

  findScriptAccountConditionByScriptAccount(scriptAccount: ScriptAccount): ScriptAccountCondition | undefined {
    return this.conditions.find(condition => condition.scriptAccount === scriptAccount);
  }

  abstract equals(state: State<Transition<any>>): boolean;
}
